(* file: babylonianSqrt.ml 
 * author: Bob Muller
 * 
 * CS1101 Computer Science I
 *
 * The Babylonian bisection algorithm for approximating the (sqrt x).
 * The algorithm determines a range lo..hi known to contain the
 * sqrt. The midpoint is taken as a guess. If the guess is adequate, it
 * is returned. If not, the algorithm repeats, this time having cut the
 * search range in half.
 *
 * See:
 * https://johncarlosbaez.wordpress.com/2011/12/02/babylon-and-the-square-root-of-2/ 
 *)

let acceptableError = 1e-8

(* closeEnough : float -> float -> bool
 *)
let closeEnough a b = abs_float(a -. b) < acceptableError

(* mySqrt : float -> int -> float
 *)
let mySqrt x n =
  let rec loop i lo hi =
    match i > n with
    | true  -> failwith "Cannot compute adequate approximation. Sorry!"
    | false ->
       let guess = (lo +. hi) /. 2.0 in
       let guessSq = guess ** 2.0
       in
       Printf.printf "try=%d, lo=%f, hi=%f, guess=%f\n" i lo hi guess; 
       match closeEnough guessSq x with
       | true  -> guess
       | false -> match guessSq > x with
		  | true  -> loop (i + 1) lo guess
		  | false -> loop (i + 1) guess hi 
  in
  loop 1 0.0 x
